#include "GLSLProgram.h"
#include "GLUtil.h"

#include <iostream>
#include <vector>

const char *vertexShaderSource =
"#version 330 core\n"

"layout(location = 0) in vec2 aPosition;"
"layout(location = 1) in vec2 aTexCoord;"
"uniform mat4 uMpMatrix;"

"out vec2 vTexCoord;"

"void main()"
"{"
"    gl_Position = uMpMatrix * vec4(aPosition, 0.0, 1.0);"
"    vTexCoord = aTexCoord;"
"}";

const char *fragmentShaderSource =
"#version 330 core\n"

"precision mediump float;"
"uniform vec3 uColor;"
"uniform sampler2D uSampler;"
"out vec4 fragColor;"
"in vec2 vTexCoord;"

"void main()"
"{"
"    vec3 c = uColor;"
"    vec2 flipped = vec2(vTexCoord.x, 1 - vTexCoord.y);"
"    fragColor = vec4(uColor, 1.0) * texture(uSampler, flipped);"
"}";

GLSLProgram::GLSLProgram()
{
}


GLSLProgram::~GLSLProgram()
{
}

GLuint GLSLProgram::CreateProgram()
{
    GLuint program = glCreateProgram();
    GLuint vShader = CreateShader(vertexShaderSource, GL_VERTEX_SHADER);
    GLuint fShader = CreateShader(fragmentShaderSource, GL_FRAGMENT_SHADER);

    glAttachShader(program, vShader);
    glAttachShader(program, fShader);
    glLinkProgram(program);
    glUseProgram(program);

    return program;
}

GLuint GLSLProgram::CreateShader(const char *shaderSource, GLenum shaderType)
{
    GLuint shader = glCreateShader(shaderType);
    glShaderSource(shader, 1, &shaderSource, nullptr);
    glCompileShader(shader);
    GLint status;
    glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
    if (status == GL_FALSE)
    {
        GLint infoLogLength = 0;
        glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLogLength);
        std::vector<GLchar> errorLog(infoLogLength);
        glGetShaderInfoLog(shader, infoLogLength, &infoLogLength, &errorLog[0]);
        glDeleteShader(shader); // Don't leak the shader.
        std::wostringstream s;
        s << "Shader failed to compile: " << &(errorLog[0]);
        TCHAR buffer[256];
        _tcscpy(buffer, s.str().c_str());
        OutErrorMsg(buffer);
    }
    return shader;
}
