#include "GLApp.h"
#include "Colors.h"
#include "GLSLProgram.h"
#include "GLTexture.h"
#include "ImageLoader.h"

// https://www.youtube.com/watch?v=6yghu5QUz50

#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtx/transform.hpp>
#include <glm/gtc/type_ptr.hpp>

glm::mat4 g_modelMatrix;
glm::mat4 g_projMatrix;
glm::mat4 g_mpMatrix;

GLint g_uMpMatrix;
GLint g_uColor;

extern "C" __declspec(dllexport) DWORD NvOptimusEnablement = 0x00000001;
extern "C" __declspec(dllexport) DWORD AmdPowerXpressRequestHighPerformance = 0x00000001;

class TestApp : public GLApp
{
public:
    TestApp(HINSTANCE hInstance);
    ~TestApp();

    // Overrides
    bool Init() override;
    void Update(float dt) override;
    void Render() override;
    LRESULT MsgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
};

TestApp::TestApp(HINSTANCE hInstance) : GLApp(hInstance)
{
}

TestApp::~TestApp()
{
}

bool TestApp::Init()
{
    if (!GLApp::Init())
        return false;

    GLfloat vertices[] = {
        -0.5f, 0.5f,
        -0.5f, -0.5f,
        0.5f, 0.5f,
        0.5f, -0.5f
    };

    GLuint vbo;
    glGenBuffers(1, &vbo);
    glBindBuffer(GL_ARRAY_BUFFER, vbo);
    glBufferData(GL_ARRAY_BUFFER, 8 * sizeof(GLfloat), vertices, GL_STATIC_DRAW);
    glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, NULL);
    glEnableVertexAttribArray(0);

    GLuint program = GLSLProgram::CreateProgram();

    g_projMatrix = glm::ortho(
        -5.0f, 5.0f,
        -5.0f, 5.0f,
        -1.0f, 1.0f);

    g_uMpMatrix = glGetUniformLocation(program, "uMpMatrix");
    if (g_uMpMatrix < 0) return OutErrorMsg(_T("Failed to get uMpMatrix"));

    g_uColor = glGetUniformLocation(program, "uColor");
    if (g_uColor < 0) return OutErrorMsg(_T("Failed to get uColor"));

    g_modelMatrix =
        glm::translate(glm::vec3(0.0f, 0.0f, 0.0f)) *
        glm::rotate(glm::radians(45.0f), glm::vec3(0.0f, 0.0f, 1.0f)) *
        glm::scale(glm::vec3(12.0f, 3.0f, 1.0f));

    g_mpMatrix = g_projMatrix * g_modelMatrix;
    glUniformMatrix4fv(g_uMpMatrix, 1, GL_FALSE, &(g_mpMatrix[0][0]));
    glUniform3f(g_uColor, 1.f, 1.0f, 1.0f);

    glClearColor(0.960f, 0.960f, 0.960f, 1.0f);

    glActiveTexture(GL_TEXTURE0);

    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);

    GLint textureLocation = glGetUniformLocation(program, "uSampler");
    glUniform1i(textureLocation, 0);

    GLfloat texCoords[] = {
        0.0f, 1.0f,
        0.0f, 0.0f,
        1.0f, 1.0f,
        1.0f, 0.0f
    };

    GLuint texture_vbo;
    glGenBuffers(1, &texture_vbo);
    glBindBuffer(GL_ARRAY_BUFFER, texture_vbo);
    glBufferData(GL_ARRAY_BUFFER, 8 * sizeof(GLfloat), texCoords, GL_STATIC_DRAW);
    glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, NULL);
    glEnableVertexAttribArray(1);

    std::string texturePath = "Assets/Textures/MyPortfolio.png";
    GLTexture newTexture = ImageLoader::loadPNG(texturePath);

    glBindTexture(GL_TEXTURE_2D, newTexture.id);

    return true;
}

void TestApp::Update(float dt)
{

}

void TestApp::Render()
{
    glViewport(0.0f, 0.0f, (float)GetWidth(), (float)GetHeight());
    glClear(GL_COLOR_BUFFER_BIT);

    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
}

LRESULT TestApp::MsgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg)
    {
        default:
            return GLApp::MsgProc(hwnd, msg, wParam, lParam);
    }
    return LRESULT();
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    TestApp app(hInstance);

    if (!app.Init())
        return 1;

    return app.Run();
}
